diff --git a/src/backend/utils/adt/varbit.c b/src/backend/utils/adt/varbit.c
index 3fa8117..e5a03b9 100644
*** a/src/backend/utils/adt/varbit.c
--- b/src/backend/utils/adt/varbit.c
***************
*** 18,23 ****
--- 18,25 ----
  
  #include "access/htup.h"
  #include "libpq/pqformat.h"
+ #include "nodes/nodeFuncs.h"
+ #include "parser/parse_clause.h"
  #include "utils/array.h"
  #include "utils/varbit.h"
  
***************
*** 646,651 **** varbit_send(PG_FUNCTION_ARGS)
--- 648,686 ----
  }
  
  /*
+  * varbit_transform()
+  * Flatten calls to our length coercion function that leave the new maximum
+  * length >= the previous maximum length.  We ignore the isExplicit argument,
+  * which only affects truncation.
+  */
+ Datum
+ varbit_transform(PG_FUNCTION_ARGS)
+ {
+ 	FuncExpr   *expr = (FuncExpr *) PG_GETARG_POINTER(0);
+ 	Node	   *typmod;
+ 	Node	   *ret = NULL;
+ 
+ 	if (!IsA(expr, FuncExpr))
+ 		PG_RETURN_POINTER(ret);
+ 
+ 	Assert(list_length(expr->args) == 3);
+ 	typmod = lsecond(expr->args);
+ 
+ 	if (IsA(typmod, Const))
+ 	{
+ 		Node	   *source = linitial(expr->args);
+ 		int32		new_typmod = DatumGetInt32(((Const *) typmod)->constvalue);
+ 		int32		old_max = exprTypmod(source);
+ 		int32		new_max = new_typmod;
+ 
+ 		if (new_max <= 0 || (old_max >= 0 && old_max <= new_max))
+ 			ret = relabel_to_typmod(source, new_typmod);
+ 	}
+ 
+ 	PG_RETURN_POINTER(ret);
+ }
+ 
+ /*
   * varbit()
   * Converts a varbit() type to a specific internal length.
   * len is the maximum bitlength specified in the column definition.
diff --git a/src/include/catalog/pg_pindex c893c3a..2a71f82 100644
*** a/src/include/catalog/pg_proc.h
--- b/src/include/catalog/pg_proc.h
***************
*** 2005,2011 **** DESCR("convert bitstring to int4");
  
  DATA(insert OID = 1685 (  bit			   PGNSP PGUID 12 1 0 0 0 f f f t f i 3 0 1560 "1560 23 16" _null_ _null_ _null_ _null_ bit _null_ _null_ _null_ ));
  DESCR("adjust bit() to typmod length");
! DATA(insert OID = 1687 (  varbit		   PGNSP PGUID 12 1 0 0 0 f f f t f i 3 0 1562 "1562 23 16" _null_ _null_ _null_ _null_ varbit _null_ _null_ _null_ ));
  DESCR("adjust varbit() to typmod length");
  
  DATA(insert OID = 1698 (  position		   PGNSP PGUID 12 1 0 0 0 f f f t f i 2 0 23 "1560 1560" _null_ _null_ _null_ _null_ bitposition _null_ _null_ _null_ ));
--- 2005,2013 ----
  
  DATA(insert OID = 1685 (  bit			   PGNSP PGUID 12 1 0 0 0 f f f t f i 3 0 1560 "1560 23 16" _null_ _null_ _null_ _null_ bit _null_ _null_ _null_ ));
  DESCR("adjust bit() to typmod length");
! DATA(insert OID = 3145 ( varbit_transform  PGNSP PGUID 12 1 0 0 0 f f f t f i 1 0 2281 "2281" _null_ _null_ _null_ _null_ varbit_transform _null_ _null_ _null_ ));
! DESCR("transform a varbit length coercion");
! DATA(insert OID = 1687 (  varbit		   PGNSP PGUID 12 1 0 0 3145 f f f t f i 3 0 1562 "1562 23 16" _null_ _null_ _null_ _null_ varbit _null_ _null_ _null_ ));
  DESCR("adjust varbit() to typmod length");
  
  DATA(insert OID = 1698 (  position		   PGNSP PGUID 12 1 0 0 0 f f f t f i 2 0 23 "1560 1560" _null_ _null_ _null_ _null_ bitposition _null_ _null_ _null_ ));
diff --git a/src/include/utils/varbiindex e329d52..5f2d8dd 100644
*** a/src/include/utils/varbit.h
--- b/src/include/utils/varbit.h
***************
*** 72,77 **** extern Datum varbit_send(PG_FUNCTION_ARGS);
--- 72,78 ----
  extern Datum varbittypmodin(PG_FUNCTION_ARGS);
  extern Datum varbittypmodout(PG_FUNCTION_ARGS);
  extern Datum bit(PG_FUNCTION_ARGS);
+ extern Datum varbit_transform(PG_FUNCTION_ARGS);
  extern Datum varbit(PG_FUNCTION_ARGS);
  extern Datum biteq(PG_FUNCTION_ARGS);
  extern Datum bitne(PG_FUNCTION_ARGS);
diff --git a/src/test/regress/expindex 301cc4b..ca83664 100644
